@@ -95,8 +95,7 @@ class NetAppApiServerTests(test.TestCase):
 
         expected_call_args_list = [mock.call('20'), mock.call(1)]
 
-        mock_invoke = self.mock_object(six, 'text_type',
-                                       mock.Mock(return_value='str'))
+        mock_invoke = self.mock_object(six, 'text_type', return_value='str')
         self.root.set_api_version(**args)
 
         self.assertEqual(expected_call_args_list, mock_invoke.call_args_list)
@@ -108,7 +107,7 @@ class NetAppApiServerTests(test.TestCase):
     def test_invoke_successfully_naapi_error(self, params):
         """Tests invoke successfully raising NaApiError"""
         self.mock_object(self.root, 'send_http_request',
-                         mock.Mock(return_value=params['result']))
+                         return_value=params['result'])
 
         self.assertRaises(netapp_api.NaApiError,
                           self.root.invoke_successfully,
@@ -116,8 +115,8 @@ class NetAppApiServerTests(test.TestCase):
 
     def test_invoke_successfully_no_error(self):
         """Tests invoke successfully with no errors"""
-        self.mock_object(self.root, 'send_http_request', mock.Mock(
-            return_value=zapi_fakes.FAKE_RESULT_SUCCESS))
+        self.mock_object(self.root, 'send_http_request',
+                         return_value=zapi_fakes.FAKE_RESULT_SUCCESS)
 
         self.assertEqual(zapi_fakes.FAKE_RESULT_SUCCESS.to_string(),
                          self.root.invoke_successfully(
@@ -130,7 +129,7 @@ class NetAppApiServerTests(test.TestCase):
         self.mock_object(self.root, '_enable_tunnel_request')
         self.mock_object(netapp_api.NaElement, 'add_child_elem')
         self.mock_object(netapp_api.NaElement, 'to_string',
-                         mock.Mock(return_value=zapi_fakes.FAKE_XML_STR))
+                         return_value=zapi_fakes.FAKE_XML_STR)
         mock_invoke = self.mock_object(urllib.request, 'Request')
 
         self.root._create_request(zapi_fakes.FAKE_NA_ELEMENT, True)
@@ -165,8 +164,7 @@ class NetAppApiServerTests(test.TestCase):
 
     def test__parse_response_no_error(self):
         """Tests parse function with appropriate response"""
-        mock_invoke = self.mock_object(etree, 'XML', mock.Mock(
-            return_value='xml'))
+        mock_invoke = self.mock_object(etree, 'XML', return_value='xml')
 
         self.root._parse_response(zapi_fakes.FAKE_XML_STR)
 
@@ -196,15 +194,16 @@ class NetAppApiServerTests(test.TestCase):
     def test_send_http_request_http_error(self):
         """Tests handling of HTTPError"""
         na_element = zapi_fakes.FAKE_NA_ELEMENT
-        self.mock_object(self.root, '_create_request', mock.Mock(
-            return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT)))
+        self.mock_object(self.root, '_create_request',
+                         return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT))
         self.mock_object(netapp_api, 'LOG')
         self.root._opener = zapi_fakes.FAKE_HTTP_OPENER
         self.mock_object(self.root, '_build_opener')
-        self.mock_object(self.root._opener, 'open', mock.Mock(
-            side_effect=urllib.error.HTTPError(url='', hdrs='',
-                                               fp=None, code='401',
-                                               msg='httperror')))
+        self.mock_object(self.root._opener, 'open',
+                         side_effect=urllib.error.HTTPError(url='', hdrs='',
+                                                            fp=None,
+                                                            code='401',
+                                                            msg='httperror'))
 
         self.assertRaises(netapp_api.NaApiError, self.root.send_http_request,
                           na_element)
@@ -212,13 +211,12 @@ class NetAppApiServerTests(test.TestCase):
     def test_send_http_request_unknown_exception(self):
         """Tests handling of Unknown Exception"""
         na_element = zapi_fakes.FAKE_NA_ELEMENT
-        self.mock_object(self.root, '_create_request', mock.Mock(
-            return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT)))
+        self.mock_object(self.root, '_create_request',
+                         return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT))
         mock_log = self.mock_object(netapp_api, 'LOG')
         self.root._opener = zapi_fakes.FAKE_HTTP_OPENER
         self.mock_object(self.root, '_build_opener')
-        self.mock_object(self.root._opener, 'open', mock.Mock(
-            side_effect=Exception))
+        self.mock_object(self.root._opener, 'open', side_effect=Exception)
 
         self.assertRaises(netapp_api.NaApiError, self.root.send_http_request,
                           na_element)
@@ -227,16 +225,14 @@ class NetAppApiServerTests(test.TestCase):
     def test_send_http_request_valid(self):
         """Tests the method send_http_request with valid parameters"""
         na_element = zapi_fakes.FAKE_NA_ELEMENT
-        self.root._trace = True
-        self.mock_object(self.root, '_create_request', mock.Mock(
-            return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT)))
+        self.mock_object(self.root, '_create_request',
+                         return_value=('abc', zapi_fakes.FAKE_NA_ELEMENT))
         self.mock_object(netapp_api, 'LOG')
         self.root._opener = zapi_fakes.FAKE_HTTP_OPENER
         self.mock_object(self.root, '_build_opener')
-        self.mock_object(self.root, '_get_result', mock.Mock(
-            return_value=zapi_fakes.FAKE_NA_ELEMENT))
-        opener_mock = self.mock_object(
-            self.root._opener, 'open', mock.Mock())
+        self.mock_object(self.root, '_get_result',
+                         return_value=zapi_fakes.FAKE_NA_ELEMENT)
+        opener_mock = self.mock_object(self.root._opener, 'open')
         opener_mock.read.side_effect = ['resp1', 'resp2']
 
         self.root.send_http_request(na_element)
@@ -245,9 +241,6 @@ class NetAppApiServerTests(test.TestCase):
 class NetAppApiElementTransTests(test.TestCase):
     """Test case for NetApp API element translations."""
 
-    def setUp(self):
-        super(NetAppApiElementTransTests, self).setUp()
-
     def test_translate_struct_dict_unique_key(self):
         """Tests if dict gets properly converted to NaElements."""
         root = netapp_api.NaElement('root')
@@ -374,10 +367,8 @@ class NetAppApiElementTransTests(test.TestCase):
     def test_getter_key_error(self):
         """Tests invalid key raises exception"""
         root = netapp_api.NaElement('root')
-        self.mock_object(root, 'get_child_by_name',
-                         mock.Mock(return_value=None))
-        self.mock_object(root, 'has_attr',
-                         mock.Mock(return_value=None))
+        self.mock_object(root, 'get_child_by_name', return_value=None)
+        self.mock_object(root, 'has_attr', return_value=None)
 
         self.assertRaises(KeyError,
                           netapp_api.NaElement.__getitem__,
@@ -395,8 +386,7 @@ class NetAppApiElementTransTests(test.TestCase):
         """Tests NaElement having no children"""
         root = netapp_api.NaElement('root')
         root.set_content('FAKE_CONTENT')
-        self.mock_object(root, 'get_child_by_name',
-                         mock.Mock(return_value=root))
+        self.mock_object(root, 'get_child_by_name', return_value=root)
 
         self.assertEqual('FAKE_CONTENT',
                          root.__getitem__('root'))
@@ -414,7 +404,7 @@ class NetAppApiElementTransTests(test.TestCase):
         root = netapp_api.NaElement('root')
         self.mock_object(netapp_api.NaElement,
                          'create_node_with_children',
-                         mock.Mock(return_value=zapi_fakes.FAKE_INVOKE_DATA))
+                         return_value=zapi_fakes.FAKE_INVOKE_DATA)
         mock_invoke = self.mock_object(root, 'add_child_elem')
 
         root.add_node_with_children('options')
@@ -424,18 +414,24 @@ class NetAppApiElementTransTests(test.TestCase):
     def test_create_node_with_children(self):
         """Tests adding a child node with its own children"""
         root = netapp_api.NaElement('root')
-        self.mock_object(root, 'add_new_child', mock.Mock(return_value='abc'))
+        self.mock_object(root, 'add_new_child', return_value='abc')
 
-        self.assertEqual(zapi_fakes.FAKE_XML1, root.create_node_with_children(
+        result_xml = str(root.create_node_with_children(
             'options', test1=zapi_fakes.FAKE_XML_STR,
-            test2=zapi_fakes.FAKE_XML_STR).to_string())
+            test2=zapi_fakes.FAKE_XML_STR))
+
+        # No ordering is guaranteed for elements in this XML.
+        self.assertTrue(result_xml.startswith("<options>"), result_xml)
+        self.assertTrue("<test1>abc</test1>" in result_xml, result_xml)
+        self.assertTrue("<test2>abc</test2>" in result_xml, result_xml)
+        self.assertTrue(result_xml.rstrip().endswith("</options>"), result_xml)
 
     def test_add_new_child(self):
         """Tests adding a child node with its own children"""
         root = netapp_api.NaElement('root')
         self.mock_object(netapp_api.NaElement,
                          '_convert_entity_refs',
-                         mock.Mock(return_value=zapi_fakes.FAKE_INVOKE_DATA))
+                         return_value=zapi_fakes.FAKE_INVOKE_DATA)
 
         root.add_new_child('options', zapi_fakes.FAKE_INVOKE_DATA)
 
@@ -472,13 +468,10 @@ class SSHUtilTests(test.TestCase):
         stdin, stdout, stderr = self._mock_ssh_channel_files(
             paramiko.ChannelFile)
         self.mock_object(ssh, 'exec_command',
-                         mock.Mock(return_value=(stdin,
-                                                 stdout,
-                                                 stderr)))
+                         return_value=(stdin, stdout, stderr))
 
         wait_on_stdout = self.mock_object(self.sshutil, '_wait_on_stdout')
-        stdout_read = self.mock_object(stdout, 'read',
-                                       mock.Mock(return_value=''))
+        stdout_read = self.mock_object(stdout, 'read', return_value='')
         self.sshutil.execute_command(ssh, 'ls')
 
         wait_on_stdout.assert_called_once_with(stdout,
@@ -502,12 +495,10 @@ class SSHUtilTests(test.TestCase):
         ssh = mock.Mock(paramiko.SSHClient)
         stdin, stdout, stderr = self._mock_ssh_channel_files(paramiko.Channel)
         stdout_read = self.mock_object(stdout.channel, 'recv',
-                                       mock.Mock(return_value=response))
+                                       return_value=response)
         stdin_write = self.mock_object(stdin, 'write')
         self.mock_object(ssh, 'exec_command',
-                         mock.Mock(return_value=(stdin,
-                                                 stdout,
-                                                 stderr)))
+                         return_value=(stdin, stdout, stderr))
 
         wait_on_stdout = self.mock_object(self.sshutil, '_wait_on_stdout')
         self.sshutil.execute_command_with_prompt(ssh, 'sudo ls',
@@ -522,11 +513,9 @@ class SSHUtilTests(test.TestCase):
         ssh = mock.Mock(paramiko.SSHClient)
         stdin, stdout, stderr = self._mock_ssh_channel_files(paramiko.Channel)
         stdout_read = self.mock_object(stdout.channel, 'recv',
-                                       mock.Mock(return_value='bad response'))
+                                       return_value='bad response')
         self.mock_object(ssh, 'exec_command',
-                         mock.Mock(return_value=(stdin,
-                                                 stdout,
-                                                 stderr)))
+                         return_value=(stdin, stdout, stderr))
 
         wait_on_stdout = self.mock_object(self.sshutil, '_wait_on_stdout')
         self.assertRaises(exception.VolumeBackendAPIException,
@@ -542,7 +531,7 @@ class SSHUtilTests(test.TestCase):
         stdout.channel = mock.Mock(paramiko.Channel)
 
         exit_status = self.mock_object(stdout.channel, 'exit_status_ready',
-                                       mock.Mock(return_value=False))
+                                       return_value=False)
         self.sshutil._wait_on_stdout(stdout, 1)
         exit_status.assert_any_call()
         self.assertGreater(exit_status.call_count, 2)
