@@ -29,16 +29,17 @@ from oslo_utils import excutils
 import six
 
 from cinder import exception
-from cinder.i18n import _, _LE, _LI, _LW
+from cinder.i18n import _
 from cinder.image import image_utils
 from cinder import interface
+from cinder.objects import fields
 from cinder import utils
 from cinder.volume.drivers.netapp.dataontap import nfs_base
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
 from cinder.volume.drivers.netapp.dataontap.utils import capabilities
 from cinder.volume.drivers.netapp.dataontap.utils import data_motion
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
-from cinder.volume.drivers.netapp.dataontap.utils import utils as cmode_utils
+from cinder.volume.drivers.netapp.dataontap.utils import utils as dot_utils
 from cinder.volume.drivers.netapp import options as na_opts
 from cinder.volume.drivers.netapp import utils as na_utils
 from cinder.volume import utils as volume_utils
@@ -57,6 +58,8 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def __init__(self, *args, **kwargs):
         super(NetAppCmodeNfsDriver, self).__init__(*args, **kwargs)
+        self.driver_name = 'NetApp_NFS_Cluster_direct'
+        self.driver_mode = 'cluster'
         self.configuration.append_config_values(na_opts.netapp_cluster_opts)
         self.failed_over_backend_name = kwargs.get('active_backend_id')
         self.failed_over = self.failed_over_backend_name is not None
@@ -70,32 +73,34 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         na_utils.check_flags(self.REQUIRED_CMODE_FLAGS, self.configuration)
 
         # cDOT API client
-        self.zapi_client = cmode_utils.get_client_for_backend(
+        self.zapi_client = dot_utils.get_client_for_backend(
             self.failed_over_backend_name or self.backend_name)
         self.vserver = self.zapi_client.vserver
 
-        # Performance monitoring library
-        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
-            self.zapi_client)
-
         # Storage service catalog
         self.ssc_library = capabilities.CapabilitiesLibrary(
             'nfs', self.vserver, self.zapi_client, self.configuration)
 
+        self.ssc_library.check_api_permissions()
+        self.using_cluster_credentials = (
+            self.ssc_library.cluster_user_supported())
+
+        # Performance monitoring library
+        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
+            self.zapi_client)
+
     def _update_zapi_client(self, backend_name):
         """Set cDOT API client for the specified config backend stanza name."""
 
-        self.zapi_client = cmode_utils.get_client_for_backend(backend_name)
+        self.zapi_client = dot_utils.get_client_for_backend(backend_name)
         self.vserver = self.zapi_client.vserver
         self.ssc_library._update_for_failover(self.zapi_client,
                                               self._get_flexvol_to_pool_map())
         ssc = self.ssc_library.get_ssc()
         self.perf_library._update_for_failover(self.zapi_client, ssc)
 
-    @utils.trace_method
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate."""
-        self.ssc_library.check_api_permissions()
         self._add_looping_tasks()
         super(NetAppCmodeNfsDriver, self).check_for_setup_error()
 
@@ -111,12 +116,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                                    loopingcalls.ONE_HOUR,
                                    loopingcalls.ONE_HOUR)
 
-        # Add the task that harvests soft-deleted QoS policy groups.
-        self.loopingcalls.add_task(
-            self.zapi_client.remove_unused_qos_policy_groups,
-            loopingcalls.ONE_MINUTE,
-            loopingcalls.ONE_MINUTE)
-
         # Add the task that runs other housekeeping tasks, such as deletion
         # of previously soft-deleted storage artifacts.
         self.loopingcalls.add_task(
@@ -126,14 +125,26 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         super(NetAppCmodeNfsDriver, self)._add_looping_tasks()
 
+    def _handle_ems_logging(self):
+        """Log autosupport messages."""
+
+        base_ems_message = dot_utils.build_ems_log_message_0(
+            self.driver_name, self.app_version)
+        self.zapi_client.send_ems_log_message(base_ems_message)
+
+        pool_ems_message = dot_utils.build_ems_log_message_1(
+            self.driver_name, self.app_version, self.vserver,
+            self._get_backing_flexvol_names(), [])
+        self.zapi_client.send_ems_log_message(pool_ems_message)
+
     def _handle_housekeeping_tasks(self):
         """Handle various cleanup activities."""
-
-        # Harvest soft-deleted QoS policy groups
-        self.zapi_client.remove_unused_qos_policy_groups()
-
         active_backend = self.failed_over_backend_name or self.backend_name
 
+        # Add the task that harvests soft-deleted QoS policy groups.
+        if self.using_cluster_credentials:
+            self.zapi_client.remove_unused_qos_policy_groups()
+
         LOG.debug("Current service state: Replication enabled: %("
                   "replication)s. Failed-Over: %(failed)s. Active Backend "
                   "ID: %(active)s",
@@ -157,11 +168,16 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             self._set_qos_policy_group_on_volume(volume, qos_policy_group_info)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Setting QoS for %s failed"), volume['id'])
+                LOG.error("Setting QoS for %s failed", volume['id'])
                 if cleanup:
                     LOG.debug("Cleaning volume %s", volume['id'])
                     self._cleanup_volume_on_failure(volume)
 
+    def _get_volume_model_update(self, volume):
+        """Provide model updates for a volume being created."""
+        if self.replication_enabled:
+            return {'replication_status': fields.ReplicationStatus.ENABLED}
+
     def _set_qos_policy_group_on_volume(self, volume, qos_policy_group_info):
         if qos_policy_group_info is None:
             return
@@ -201,9 +217,8 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         LOG.debug('Updating volume stats')
         data = {}
-        netapp_backend = 'NetApp_NFS_Cluster_direct'
         backend_name = self.configuration.safe_get('volume_backend_name')
-        data['volume_backend_name'] = backend_name or netapp_backend
+        data['volume_backend_name'] = backend_name or self.driver_name
         data['vendor_name'] = 'NetApp'
         data['driver_version'] = self.VERSION
         data['storage_protocol'] = 'nfs'
@@ -216,7 +231,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         data['replication_enabled'] = self.replication_enabled
 
         self._spawn_clean_cache_job()
-        self.zapi_client.provide_ems(self, netapp_backend, self._app_version)
         self._stats = data
 
     def _get_pool_stats(self, filter_function=None, goodness_function=None):
@@ -234,12 +248,18 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         if not ssc:
             return pools
 
-        # Get up-to-date node utilization metrics just once
-        self.perf_library.update_performance_cache(ssc)
+        # Utilization and performance metrics require cluster-scoped
+        # credentials
+        if self.using_cluster_credentials:
+            # Get up-to-date node utilization metrics just once
+            self.perf_library.update_performance_cache(ssc)
 
-        # Get up-to-date aggregate capacities just once
-        aggregates = self.ssc_library.get_ssc_aggregates()
-        aggr_capacities = self.zapi_client.get_aggregate_capacities(aggregates)
+            # Get up-to-date aggregate capacities just once
+            aggregates = self.ssc_library.get_ssc_aggregates()
+            aggr_capacities = self.zapi_client.get_aggregate_capacities(
+                aggregates)
+        else:
+            aggr_capacities = {}
 
         for ssc_vol_name, ssc_vol_info in ssc.items():
 
@@ -249,15 +269,25 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             pool.update(ssc_vol_info)
 
             # Add driver capabilities and config info
-            pool['QoS_support'] = True
+            pool['QoS_support'] = self.using_cluster_credentials
             pool['consistencygroup_support'] = True
+            pool['consistent_group_snapshot_enabled'] = True
             pool['multiattach'] = True
+            pool['online_extend_support'] = False
 
             # Add up-to-date capacity info
             nfs_share = ssc_vol_info['pool_name']
             capacity = self._get_share_capacity_info(nfs_share)
             pool.update(capacity)
 
+            if self.using_cluster_credentials:
+                dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
+                    ssc_vol_name)
+            else:
+                dedupe_used = 0.0
+            pool['netapp_dedupe_used_percent'] = na_utils.round_down(
+                dedupe_used)
+
             aggregate_name = ssc_vol_info.get('netapp_aggregate')
             aggr_capacity = aggr_capacities.get(aggregate_name, {})
             pool['netapp_aggregate_used_percent'] = aggr_capacity.get(
@@ -301,8 +331,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             address = na_utils.resolve_hostname(host)
 
             if address not in vserver_addresses:
-                msg = _LW('Address not found for NFS share %s.')
-                LOG.warning(msg, share)
+                LOG.warning('Address not found for NFS share %s.', share)
                 continue
 
             try:
@@ -310,8 +339,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                     flexvol_path=junction_path)
                 pools[flexvol['name']] = {'pool_name': share}
             except exception.VolumeBackendAPIException:
-                msg = _LE('Flexvol not found for NFS share %s.')
-                LOG.exception(msg, share)
+                LOG.exception('Flexvol not found for NFS share %s.', share)
 
         return pools
 
@@ -357,16 +385,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
     def _is_share_clone_compatible(self, volume, share):
         """Checks if share is compatible with volume to host its clone."""
         flexvol_name = self._get_flexvol_name_for_share(share)
-        thin = self._is_volume_thin_provisioned(flexvol_name)
-        return (
-            self._share_has_space_for_clone(share, volume['size'], thin) and
-            self._is_share_vol_type_match(volume, share, flexvol_name)
-        )
-
-    def _is_volume_thin_provisioned(self, flexvol_name):
-        """Checks if a flexvol is thin (sparse file or thin provisioned)."""
-        ssc_info = self.ssc_library.get_ssc_for_flexvol(flexvol_name)
-        return ssc_info.get('thin_provisioning_support') or False
+        return self._is_share_vol_type_match(volume, share, flexvol_name)
 
     def _is_share_vol_type_match(self, volume, share, flexvol_name):
         """Checks if share matches volume type."""
@@ -385,7 +404,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 return ssc_vol_name
         return None
 
-    @utils.trace_method
     def delete_volume(self, volume):
         """Deletes a logical volume."""
         self._delete_backing_file_for_volume(volume)
@@ -405,23 +423,25 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             LOG.debug('Deleting backing file for volume %s.', volume['id'])
             self._delete_file(volume['id'], volume['name'])
         except Exception:
-            LOG.exception(_LE('Could not delete volume %s on backend, '
-                              'falling back to exec of "rm" command.'),
+            LOG.exception('Could not delete volume %s on backend, '
+                          'falling back to exec of "rm" command.',
                           volume['id'])
             try:
                 super(NetAppCmodeNfsDriver, self).delete_volume(volume)
             except Exception:
-                LOG.exception(_LE('Exec of "rm" command on backing file for '
-                                  '%s was unsuccessful.'), volume['id'])
+                LOG.exception('Exec of "rm" command on backing file for '
+                              '%s was unsuccessful.', volume['id'])
 
     def _delete_file(self, file_id, file_name):
-        (_vserver, flexvol) = self._get_export_ip_path(volume_id=file_id)
-        path_on_backend = '/vol' + flexvol + '/' + file_name
+        (host_ip, junction_path) = self._get_export_ip_path(volume_id=file_id)
+        vserver = self._get_vserver_for_ip(host_ip)
+        flexvol = self.zapi_client.get_vol_by_junc_vserver(
+            vserver, junction_path)
+        path_on_backend = '/vol/' + flexvol + '/' + file_name
         LOG.debug('Attempting to delete file %(path)s for ID %(file_id)s on '
                   'backend.', {'path': path_on_backend, 'file_id': file_id})
         self.zapi_client.delete_file(path_on_backend)
 
-    @utils.trace_method
     def delete_snapshot(self, snapshot):
         """Deletes a snapshot."""
         self._delete_backing_file_for_snapshot(snapshot)
@@ -432,48 +452,15 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             LOG.debug('Deleting backing file for snapshot %s.', snapshot['id'])
             self._delete_file(snapshot['volume_id'], snapshot['name'])
         except Exception:
-            LOG.exception(_LE('Could not delete snapshot %s on backend, '
-                              'falling back to exec of "rm" command.'),
+            LOG.exception('Could not delete snapshot %s on backend, '
+                          'falling back to exec of "rm" command.',
                           snapshot['id'])
             try:
                 # delete_file_from_share
                 super(NetAppCmodeNfsDriver, self).delete_snapshot(snapshot)
             except Exception:
-                LOG.exception(_LE('Exec of "rm" command on backing file for'
-                                  ' %s was unsuccessful.'), snapshot['id'])
-
-    @utils.trace_method
-    def copy_image_to_volume(self, context, volume, image_service, image_id):
-        """Fetch the image from image_service and write it to the volume."""
-        copy_success = False
-        try:
-            major, minor = self.zapi_client.get_ontapi_version()
-            col_path = self.configuration.netapp_copyoffload_tool_path
-            # Search the local image cache before attempting copy offload
-            cache_result = self._find_image_in_cache(image_id)
-            if cache_result:
-                copy_success = self._copy_from_cache(volume, image_id,
-                                                     cache_result)
-                if copy_success:
-                    LOG.info(_LI('Copied image %(img)s to volume %(vol)s '
-                                 'using local image cache.'),
-                             {'img': image_id, 'vol': volume['id']})
-            # Image cache was not present, attempt copy offload workflow
-            if (not copy_success and col_path and
-                    major == 1 and minor >= 20):
-                LOG.debug('No result found in image cache')
-                self._copy_from_img_service(context, volume, image_service,
-                                            image_id)
-                LOG.info(_LI('Copied image %(img)s to volume %(vol)s using'
-                             ' copy offload workflow.'),
-                         {'img': image_id, 'vol': volume['id']})
-                copy_success = True
-        except Exception as e:
-            LOG.exception(_LE('Copy offload workflow unsuccessful. %s'), e)
-        finally:
-            if not copy_success:
-                super(NetAppCmodeNfsDriver, self).copy_image_to_volume(
-                    context, volume, image_service, image_id)
+                LOG.exception('Exec of "rm" command on backing file for'
+                              ' %s was unsuccessful.', snapshot['id'])
 
     def _get_ip_verify_on_cluster(self, host):
         """Verifies if host on same cluster and returns ip."""
@@ -486,31 +473,29 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _copy_from_cache(self, volume, image_id, cache_result):
         """Try copying image file_name from cached file_name."""
-        LOG.debug("Trying copy from cache using copy offload.")
         copied = False
         cache_copy, found_local = self._find_image_location(cache_result,
-                                                            volume['id'])
+                                                            volume)
 
         try:
             if found_local:
+                LOG.debug("Trying copy from cache using cloning.")
                 (nfs_share, file_name) = cache_copy
                 self._clone_file_dst_exists(
                     nfs_share, file_name, volume['name'], dest_exists=True)
                 LOG.debug("Copied image from cache to volume %s using "
                           "cloning.", volume['id'])
                 copied = True
-            elif cache_copy:
+            elif (cache_copy and
+                  self.configuration.netapp_copyoffload_tool_path):
+                LOG.debug("Trying copy from cache using copy offload.")
                 self._copy_from_remote_cache(volume, image_id, cache_copy)
                 copied = True
-
-            if copied:
-                self._post_clone_image(volume)
-
-        except Exception as e:
-            LOG.exception(_LE('Error in workflow copy from cache. %s.'), e)
+        except Exception:
+            LOG.exception('Error in workflow copy from cache.')
         return copied
 
-    def _find_image_location(self, cache_result, volume_id):
+    def _find_image_location(self, cache_result, volume):
         """Finds the location of a cached image.
 
         Returns image location local to the NFS share, that matches the
@@ -520,7 +505,10 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         found_local_copy = False
         cache_copy = None
-        provider_location = self._get_provider_location(volume_id)
+
+        provider_location = volume_utils.extract_host(volume['host'],
+                                                      level='pool')
+
         for res in cache_result:
             (share, file_name) = res
             if share == provider_location:
@@ -558,10 +546,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         return src_ip, src_path
 
     def _get_destination_ip_and_path(self, volume):
-        dest_ip = self._get_ip_verify_on_cluster(
-            self._get_host_ip(volume['id']))
-        dest_path = os.path.join(self._get_export_path(
-            volume['id']), volume['name'])
+        share = volume_utils.extract_host(volume['host'], level='pool')
+        share_ip_and_path = share.split(":")
+        dest_ip = self._get_ip_verify_on_cluster(share_ip_and_path[0])
+        dest_path = os.path.join(share_ip_and_path[1], volume['name'])
+
         return dest_ip, dest_path
 
     def _clone_file_dst_exists(self, share, src_name, dst_name,
@@ -574,11 +563,14 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
     def _copy_from_img_service(self, context, volume, image_service,
                                image_id):
         """Copies from the image service using copy offload."""
+
         LOG.debug("Trying copy from image service using copy offload.")
         image_loc = image_service.get_location(context, image_id)
         locations = self._construct_image_nfs_url(image_loc)
         src_ip = None
         selected_loc = None
+        cloned = False
+
         # this will match the first location that has a valid IP on cluster
         for location in locations:
             conn, dr = self._check_get_nfs_path_segs(location)
@@ -593,32 +585,31 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             raise exception.NotFound(_("Source host details not found."))
         (__, ___, img_file) = selected_loc.rpartition('/')
         src_path = os.path.join(dr, img_file)
-        dst_ip = self._get_ip_verify_on_cluster(self._get_host_ip(
-            volume['id']))
+
+        dst_ip, vol_path = self._get_destination_ip_and_path(volume)
+        share_path = vol_path.rsplit("/", 1)[0]
+        dst_share = dst_ip + ':' + share_path
+
         # tmp file is required to deal with img formats
         tmp_img_file = six.text_type(uuid.uuid4())
         col_path = self.configuration.netapp_copyoffload_tool_path
         img_info = image_service.show(context, image_id)
-        dst_share = self._get_provider_location(volume['id'])
         self._check_share_can_hold_size(dst_share, img_info['size'])
         run_as_root = self._execute_as_root
 
         dst_dir = self._get_mount_point_for_share(dst_share)
         dst_img_local = os.path.join(dst_dir, tmp_img_file)
+
         try:
-            # If src and dst share not equal
-            if (('%s:%s' % (src_ip, dr)) !=
-                    ('%s:%s' % (dst_ip, self._get_export_path(volume['id'])))):
-                dst_img_serv_path = os.path.join(
-                    self._get_export_path(volume['id']), tmp_img_file)
-                # Always run copy offload as regular user, it's sufficient
-                # and rootwrap doesn't allow copy offload to run as root
-                # anyways.
-                self._execute(col_path, src_ip, dst_ip, src_path,
-                              dst_img_serv_path, run_as_root=False,
-                              check_exit_code=0)
-            else:
-                self._clone_file_dst_exists(dst_share, img_file, tmp_img_file)
+            dst_img_serv_path = os.path.join(
+                share_path, tmp_img_file)
+            # Always run copy offload as regular user, it's sufficient
+            # and rootwrap doesn't allow copy offload to run as root
+            # anyways.
+            self._execute(col_path, src_ip, dst_ip, src_path,
+                          dst_img_serv_path, run_as_root=False,
+                          check_exit_code=0)
+
             self._discover_file_till_timeout(dst_img_local, timeout=120)
             LOG.debug('Copied image %(img)s to tmp file %(tmp)s.',
                       {'img': image_id, 'tmp': tmp_img_file})
@@ -660,12 +651,13 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 finally:
                     if os.path.exists(dst_img_conv_local):
                         self._delete_file_at_path(dst_img_conv_local)
-            self._post_clone_image(volume)
+            cloned = True
         finally:
             if os.path.exists(dst_img_local):
                 self._delete_file_at_path(dst_img_local)
 
-    @utils.trace_method
+        return cloned
+
     def unmanage(self, volume):
         """Removes the specified volume from Cinder management.
 
@@ -687,7 +679,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         super(NetAppCmodeNfsDriver, self).unmanage(volume)
 
-    def failover_host(self, context, volumes, secondary_id=None):
+    def failover_host(self, context, volumes, secondary_id=None, groups=None):
         """Failover a backend to a secondary replication target."""
 
         return self._failover_host(volumes, secondary_id=secondary_id)
@@ -710,9 +702,8 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         return flexvols
 
-    @utils.trace_method
-    def delete_cgsnapshot(self, context, cgsnapshot, snapshots):
-        """Delete files backing each snapshot in the cgsnapshot.
+    def delete_group_snapshot(self, context, group_snapshot, snapshots):
+        """Delete files backing each snapshot in the group snapshot.
 
         :return: An implicit update of snapshot models that the manager will
                  interpret and subsequently set the model state to deleted.
@@ -722,3 +713,161 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             LOG.debug("Snapshot %s deletion successful", snapshot['name'])
 
         return None, None
+
+    def create_group(self, context, group):
+        """Driver entry point for creating a generic volume group.
+
+        ONTAP does not maintain an actual group construct. As a result, no
+        communtication to the backend is necessary for generic volume group
+        creation.
+
+        :returns: Hard-coded model update for generic volume group model.
+        """
+        model_update = {'status': fields.GroupStatus.AVAILABLE}
+        return model_update
+
+    def delete_group(self, context, group, volumes):
+        """Driver entry point for deleting a generic volume group.
+
+        :returns: Updated group model and list of volume models for the volumes
+                 that were deleted.
+        """
+        model_update = {'status': fields.GroupStatus.DELETED}
+        volumes_model_update = []
+        for volume in volumes:
+            try:
+                self._delete_file(volume['id'], volume['name'])
+                volumes_model_update.append(
+                    {'id': volume['id'], 'status': 'deleted'})
+            except Exception:
+                volumes_model_update.append(
+                    {'id': volume['id'],
+                     'status': fields.GroupStatus.ERROR_DELETING})
+                LOG.exception("Volume %(vol)s in the group could not be "
+                              "deleted.", {'vol': volume})
+        return model_update, volumes_model_update
+
+    def update_group(self, context, group, add_volumes=None,
+                     remove_volumes=None):
+        """Driver entry point for updating a generic volume group.
+
+        Since no actual group construct is ever created in ONTAP, it is not
+        necessary to update any metadata on the backend. Since this is a NO-OP,
+        there is guaranteed to be no change in any of the volumes' statuses.
+        """
+
+        return None, None, None
+
+    def create_group_snapshot(self, context, group_snapshot, snapshots):
+        """Creates a Cinder group snapshot object.
+
+        The Cinder group snapshot object is created by making use of an ONTAP
+        consistency group snapshot in order to provide write-order consistency
+        for a set of flexvols snapshots. First, a list of the flexvols backing
+        the given Cinder group must be gathered. An ONTAP group-snapshot of
+        these flexvols will create a snapshot copy of all the Cinder volumes in
+        the generic volume group. For each Cinder volume in the group, it is
+        then necessary to clone its backing file from the ONTAP cg-snapshot.
+        The naming convention used to for the clones is what indicates the
+        clone's role as a Cinder snapshot and its inclusion in a Cinder group.
+        The ONTAP cg-snapshot of the flexvols is deleted after the cloning
+        operation is completed.
+
+        :returns: An implicit update for the group snapshot and snapshot models
+                 that is then used by the manager to set the models to
+                 available.
+        """
+        try:
+            if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
+                self._create_consistent_group_snapshot(group_snapshot,
+                                                       snapshots)
+            else:
+                for snapshot in snapshots:
+                    self._clone_backing_file_for_volume(
+                        snapshot['volume_name'], snapshot['name'],
+                        snapshot['volume_id'], is_snapshot=True)
+        except Exception as ex:
+            err_msg = (_("Create group snapshot failed (%s).") % ex)
+            LOG.exception(err_msg, resource=group_snapshot)
+            raise exception.NetAppDriverException(err_msg)
+
+        return None, None
+
+    def _create_consistent_group_snapshot(self, group_snapshot, snapshots):
+        hosts = [snapshot['volume']['host'] for snapshot in snapshots]
+        flexvols = self._get_flexvol_names_from_hosts(hosts)
+
+        # Create snapshot for backing flexvol
+        self.zapi_client.create_cg_snapshot(flexvols, group_snapshot['id'])
+
+        # Start clone process for snapshot files
+        for snapshot in snapshots:
+            self._clone_backing_file_for_volume(
+                snapshot['volume']['name'], snapshot['name'],
+                snapshot['volume']['id'], source_snapshot=group_snapshot['id'])
+
+        # Delete backing flexvol snapshots
+        for flexvol_name in flexvols:
+            try:
+                self.zapi_client.wait_for_busy_snapshot(
+                    flexvol_name, group_snapshot['id'])
+                self.zapi_client.delete_snapshot(
+                    flexvol_name, group_snapshot['id'])
+            except exception.SnapshotIsBusy:
+                self.zapi_client.mark_snapshot_for_deletion(
+                    flexvol_name, group_snapshot['id'])
+
+    def create_group_from_src(self, context, group, volumes,
+                              group_snapshot=None, sorted_snapshots=None,
+                              source_group=None, sorted_source_vols=None):
+        """Creates a group from a group snapshot or a group of cinder vols.
+
+        :returns: An implicit update for the volumes model that is
+                 interpreted by the manager as a successful operation.
+        """
+        LOG.debug("VOLUMES %s ", ', '.join([vol['id'] for vol in volumes]))
+        model_update = None
+        volumes_model_update = []
+
+        if group_snapshot:
+            vols = zip(volumes, sorted_snapshots)
+
+            for volume, snapshot in vols:
+                update = self.create_volume_from_snapshot(
+                    volume, snapshot)
+                update['id'] = volume['id']
+                volumes_model_update.append(update)
+
+        elif source_group and sorted_source_vols:
+            hosts = [source_vol['host'] for source_vol in sorted_source_vols]
+            flexvols = self._get_flexvol_names_from_hosts(hosts)
+
+            # Create snapshot for backing flexvol
+            snapshot_name = 'snapshot-temp-' + source_group['id']
+            self.zapi_client.create_cg_snapshot(flexvols, snapshot_name)
+
+            # Start clone process for new volumes
+            vols = zip(volumes, sorted_source_vols)
+            for volume, source_vol in vols:
+                self._clone_backing_file_for_volume(
+                    source_vol['name'], volume['name'],
+                    source_vol['id'], source_snapshot=snapshot_name)
+                volume_model_update = (
+                    self._get_volume_model_update(volume) or {})
+                volume_model_update.update({
+                    'id': volume['id'],
+                    'provider_location': source_vol['provider_location'],
+                })
+                volumes_model_update.append(volume_model_update)
+
+            # Delete backing flexvol snapshots
+            for flexvol_name in flexvols:
+                self.zapi_client.wait_for_busy_snapshot(
+                    flexvol_name, snapshot_name)
+                self.zapi_client.delete_snapshot(flexvol_name, snapshot_name)
+        else:
+            LOG.error("Unexpected set of parameters received when "
+                      "creating group from source.")
+            model_update = {'status': fields.GroupStatus.ERROR}
+
+        return model_update, volumes_model_update
